"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class EmployeeCheckNumberMappingTypedDict(TypedDict):
    employee_uuid: NotRequired[str]
    r"""The UUID for an employee"""
    check_number: NotRequired[float]
    r"""The check number for the relevant employee"""


class EmployeeCheckNumberMapping(BaseModel):
    employee_uuid: Optional[str] = None
    r"""The UUID for an employee"""

    check_number: Optional[float] = None
    r"""The check number for the relevant employee"""


class PayrollCheckTypedDict(TypedDict):
    r"""Example response"""

    payroll_uuid: NotRequired[str]
    r"""A unique identifier of the payroll."""
    printing_format: NotRequired[str]
    r"""The format the checks will be printed."""
    starting_check_number: NotRequired[str]
    r"""The starting check number for the checks being printed."""
    request_uuid: NotRequired[str]
    r"""A unique identifier of the Generated Document request"""
    status: NotRequired[str]
    r"""Current status of the Generated Document"""
    employee_check_number_mapping: NotRequired[
        List[EmployeeCheckNumberMappingTypedDict]
    ]
    r"""An array of mapping employee uuids to their check numbers"""


class PayrollCheck(BaseModel):
    r"""Example response"""

    payroll_uuid: Optional[str] = None
    r"""A unique identifier of the payroll."""

    printing_format: Optional[str] = None
    r"""The format the checks will be printed."""

    starting_check_number: Optional[str] = None
    r"""The starting check number for the checks being printed."""

    request_uuid: Optional[str] = None
    r"""A unique identifier of the Generated Document request"""

    status: Optional[str] = None
    r"""Current status of the Generated Document"""

    employee_check_number_mapping: Optional[List[EmployeeCheckNumberMapping]] = None
    r"""An array of mapping employee uuids to their check numbers"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EmployeeOnboardingDocumentTypedDict(TypedDict):
    r"""Configuration for an employee onboarding documents during onboarding"""

    i9_document: NotRequired[str]
    r"""Whether to include Form I-9 for an employee during onboarding"""


class EmployeeOnboardingDocument(BaseModel):
    r"""Configuration for an employee onboarding documents during onboarding"""

    i9_document: Optional[str] = None
    r"""Whether to include Form I-9 for an employee during onboarding"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from gusto_embedded.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ContractorPaymentPaymentMethod(str, Enum):
    r"""The payment method."""

    DIRECT_DEPOSIT = "Direct Deposit"
    CHECK = "Check"
    HISTORICAL_PAYMENT = "Historical Payment"
    CORRECTION_PAYMENT = "Correction Payment"


class ContractorPaymentStatus(str, Enum):
    r"""Contractor payment status"""

    FUNDED = "Funded"
    UNFUNDED = "Unfunded"


class ContractorPaymentWageType(str, Enum):
    r"""The wage type for the payment."""

    HOURLY = "Hourly"
    FIXED = "Fixed"


class ContractorPaymentTypedDict(TypedDict):
    r"""The representation of a single contractor payment."""

    uuid: str
    r"""The unique identifier of the contractor payment in Gusto."""
    contractor_uuid: NotRequired[str]
    r"""The UUID of the contractor."""
    bonus: NotRequired[str]
    r"""The bonus amount in the payment."""
    date_: NotRequired[str]
    r"""The payment date."""
    hours: NotRequired[str]
    r"""The number of hours worked for the payment."""
    payment_method: NotRequired[ContractorPaymentPaymentMethod]
    r"""The payment method."""
    reimbursement: NotRequired[str]
    r"""The reimbursement amount in the payment."""
    status: NotRequired[ContractorPaymentStatus]
    r"""Contractor payment status"""
    hourly_rate: NotRequired[str]
    r"""The rate per hour worked for the payment."""
    may_cancel: NotRequired[bool]
    r"""Determine if the contractor payment can be cancelled."""
    wage: NotRequired[str]
    r"""The fixed wage of the payment, regardless of hours worked."""
    wage_type: NotRequired[ContractorPaymentWageType]
    r"""The wage type for the payment."""
    wage_total: NotRequired[str]
    r"""(hours * hourly_rate) + wage + bonus"""


class ContractorPayment(BaseModel):
    r"""The representation of a single contractor payment."""

    uuid: str
    r"""The unique identifier of the contractor payment in Gusto."""

    contractor_uuid: Optional[str] = None
    r"""The UUID of the contractor."""

    bonus: Optional[str] = None
    r"""The bonus amount in the payment."""

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None
    r"""The payment date."""

    hours: Optional[str] = None
    r"""The number of hours worked for the payment."""

    payment_method: Optional[ContractorPaymentPaymentMethod] = None
    r"""The payment method."""

    reimbursement: Optional[str] = None
    r"""The reimbursement amount in the payment."""

    status: Optional[ContractorPaymentStatus] = None
    r"""Contractor payment status"""

    hourly_rate: Optional[str] = None
    r"""The rate per hour worked for the payment."""

    may_cancel: Optional[bool] = None
    r"""Determine if the contractor payment can be cancelled."""

    wage: Optional[str] = None
    r"""The fixed wage of the payment, regardless of hours worked."""

    wage_type: Optional[ContractorPaymentWageType] = None
    r"""The wage type for the payment."""

    wage_total: Optional[str] = None
    r"""(hours * hourly_rate) + wage + bonus"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gusto_embedded import models, utils
from gusto_embedded._hooks import HookContext
from gusto_embedded.types import OptionalNullable, UNSET
from gusto_embedded.utils import get_security_from_env
from typing import Any, Mapping, Optional, Union


class Invoices(BaseSDK):
    def get(
        self,
        *,
        security: Union[
            models.GetInvoicesInvoicePeriodSecurity,
            models.GetInvoicesInvoicePeriodSecurityTypedDict,
        ],
        invoice_period: str,
        page: Optional[int] = None,
        per: Optional[int] = None,
        company_uuids: Optional[str] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InvoiceData:
        r"""Retrieve invoicing data for companies

        Retrieve data for active companies used to calculate invoices for Gusto Embedded Payroll. A company is considered active for an invoice period if they are an active partner managed company, have run payroll or created contractor payments since becoming a partner managed company, and are not suspended at any point during the invoice period.  This endpoint forces pagination, with 100 results returned at a time. You can learn more about our pagination here: [pagination guide](https://docs.gusto.com/embedded-payroll/docs/pagination)

        > 📘 System Access Authentication
        >
        > This endpoint uses the [Bearer Auth scheme with the system-level access token in the HTTP Authorization header](https://docs.gusto.com/embedded-payroll/docs/system-access)

        scope: `invoices:read`

        :param security:
        :param invoice_period: The month we are calculating the invoice for. Must be in YYYY-MM format
        :param page: The page that is requested. When unspecified, will load all objects unless endpoint forces pagination.
        :param per: Number of objects per page. For majority of endpoints will default to 25
        :param company_uuids: Filter companies returned in the active_companies response, will return an error if company not active during provided invoice period. i.e. `?company_uuids=781922d8-e780-4b6b-bf74-ee303166d022,bbbca930-7322-491c-ba7f-98707a52a9c5`
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetInvoicesInvoicePeriodRequest(
            invoice_period=invoice_period,
            page=page,
            per=per,
            company_uuids=company_uuids,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/invoices/{invoice_period}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetInvoicesInvoicePeriodSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-invoices-invoice-period",
                oauth2_scopes=[],
                security_source=get_security_from_env(security, models.Security),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.InvoiceData)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        security: Union[
            models.GetInvoicesInvoicePeriodSecurity,
            models.GetInvoicesInvoicePeriodSecurityTypedDict,
        ],
        invoice_period: str,
        page: Optional[int] = None,
        per: Optional[int] = None,
        company_uuids: Optional[str] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InvoiceData:
        r"""Retrieve invoicing data for companies

        Retrieve data for active companies used to calculate invoices for Gusto Embedded Payroll. A company is considered active for an invoice period if they are an active partner managed company, have run payroll or created contractor payments since becoming a partner managed company, and are not suspended at any point during the invoice period.  This endpoint forces pagination, with 100 results returned at a time. You can learn more about our pagination here: [pagination guide](https://docs.gusto.com/embedded-payroll/docs/pagination)

        > 📘 System Access Authentication
        >
        > This endpoint uses the [Bearer Auth scheme with the system-level access token in the HTTP Authorization header](https://docs.gusto.com/embedded-payroll/docs/system-access)

        scope: `invoices:read`

        :param security:
        :param invoice_period: The month we are calculating the invoice for. Must be in YYYY-MM format
        :param page: The page that is requested. When unspecified, will load all objects unless endpoint forces pagination.
        :param per: Number of objects per page. For majority of endpoints will default to 25
        :param company_uuids: Filter companies returned in the active_companies response, will return an error if company not active during provided invoice period. i.e. `?company_uuids=781922d8-e780-4b6b-bf74-ee303166d022,bbbca930-7322-491c-ba7f-98707a52a9c5`
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetInvoicesInvoicePeriodRequest(
            invoice_period=invoice_period,
            page=page,
            per=per,
            company_uuids=company_uuids,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/invoices/{invoice_period}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetInvoicesInvoicePeriodSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-invoices-invoice-period",
                oauth2_scopes=[],
                security_source=get_security_from_env(security, models.Security),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.InvoiceData)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gusto_embedded import models, utils
from gusto_embedded._hooks import HookContext
from gusto_embedded.types import OptionalNullable, UNSET
from gusto_embedded.utils import get_security_from_env
from typing import Any, Mapping, Optional


class Notifications(BaseSDK):
    def get_details(
        self,
        *,
        notification_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Notification:
        r"""Get a notification's details

        Upon receiving a notification webhook event, use this endpoint to fetch the notification's details. The notification details include basic suggested content that can help you build notifications in your platform.

        Note: partners are responsible for the delivery and any custom state management of notifications in their application. Refer to our [partner notification guide](https://docs.gusto.com/embedded-payroll/docs/partner-notifications) for more details.

        If the notification UUID is not found, the response will be 404 Not Found. If the notification's supporting data is no longer valid, the response will be 422 Unprocessable Entity.

        scope: `notifications:read`

        :param notification_uuid: The notification entity_uuid
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetNotificationsNotificationUUIDRequest(
            notification_uuid=notification_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/notifications/{notification_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-notifications-notification_uuid",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Notification)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_details_async(
        self,
        *,
        notification_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Notification:
        r"""Get a notification's details

        Upon receiving a notification webhook event, use this endpoint to fetch the notification's details. The notification details include basic suggested content that can help you build notifications in your platform.

        Note: partners are responsible for the delivery and any custom state management of notifications in their application. Refer to our [partner notification guide](https://docs.gusto.com/embedded-payroll/docs/partner-notifications) for more details.

        If the notification UUID is not found, the response will be 404 Not Found. If the notification's supporting data is no longer valid, the response will be 422 Unprocessable Entity.

        scope: `notifications:read`

        :param notification_uuid: The notification entity_uuid
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetNotificationsNotificationUUIDRequest(
            notification_uuid=notification_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/notifications/{notification_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-notifications-notification_uuid",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Notification)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import date
from gusto_embedded import models, utils
from gusto_embedded._hooks import HookContext
from gusto_embedded.types import OptionalNullable, UNSET
from gusto_embedded.utils import get_security_from_env
from typing import Any, List, Mapping, Optional


class Reports(BaseSDK):
    def create_custom(
        self,
        *,
        company_uuid: str,
        columns: List[models.Columns],
        groupings: List[models.Groupings],
        file_type: models.FileType,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        custom_name: Optional[str] = None,
        with_totals: Optional[bool] = False,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        dismissed_start_date: Optional[date] = None,
        dismissed_end_date: Optional[date] = None,
        payment_method: Optional[
            models.PostCompaniesCompanyUUIDReportsPaymentMethod
        ] = None,
        employment_type: Optional[models.EmploymentType] = None,
        employment_status: Optional[
            models.PostCompaniesCompanyUUIDReportsEmploymentStatus
        ] = None,
        employee_uuids: Optional[List[str]] = None,
        department_uuids: Optional[List[str]] = None,
        work_address_uuids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateReport:
        r"""Create a custom report

        Create a custom report for a company. This endpoint initiates creating a custom report with custom columns, groupings, and filters. The `request_uuid` in the response can then be used to poll for the status and report URL upon completion using the report GET endpoint.

        scope: `company_reports:write`

        :param company_uuid: The UUID of the company
        :param columns: Columns to include in the report
        :param groupings: How to group the report
        :param file_type: The type of file to generate
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param custom_name: The title of the report
        :param with_totals: Whether to include subtotals and grand totals in the report
        :param start_date: Start date of data to filter by
        :param end_date: End date of data to filter by
        :param dismissed_start_date: Dismissed start date of employees to filter by
        :param dismissed_end_date: Dismissed end date of employees to filter by
        :param payment_method: Payment method to filter by
        :param employment_type: Employee employment type to filter by
        :param employment_status: Employee employment status to filter by
        :param employee_uuids: Employees to filter by
        :param department_uuids: Departments to filter by
        :param work_address_uuids: Work addresses to filter by
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostCompaniesCompanyUUIDReportsRequest(
            company_uuid=company_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostCompaniesCompanyUUIDReportsRequestBody(
                columns=columns,
                groupings=groupings,
                custom_name=custom_name,
                file_type=file_type,
                with_totals=with_totals,
                start_date=start_date,
                end_date=end_date,
                dismissed_start_date=dismissed_start_date,
                dismissed_end_date=dismissed_end_date,
                payment_method=payment_method,
                employment_type=employment_type,
                employment_status=employment_status,
                employee_uuids=employee_uuids,
                department_uuids=department_uuids,
                work_address_uuids=work_address_uuids,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/companies/{company_uuid}/reports",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostCompaniesCompanyUUIDReportsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-companies-company_uuid-reports",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CreateReport)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_custom_async(
        self,
        *,
        company_uuid: str,
        columns: List[models.Columns],
        groupings: List[models.Groupings],
        file_type: models.FileType,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        custom_name: Optional[str] = None,
        with_totals: Optional[bool] = False,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        dismissed_start_date: Optional[date] = None,
        dismissed_end_date: Optional[date] = None,
        payment_method: Optional[
            models.PostCompaniesCompanyUUIDReportsPaymentMethod
        ] = None,
        employment_type: Optional[models.EmploymentType] = None,
        employment_status: Optional[
            models.PostCompaniesCompanyUUIDReportsEmploymentStatus
        ] = None,
        employee_uuids: Optional[List[str]] = None,
        department_uuids: Optional[List[str]] = None,
        work_address_uuids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateReport:
        r"""Create a custom report

        Create a custom report for a company. This endpoint initiates creating a custom report with custom columns, groupings, and filters. The `request_uuid` in the response can then be used to poll for the status and report URL upon completion using the report GET endpoint.

        scope: `company_reports:write`

        :param company_uuid: The UUID of the company
        :param columns: Columns to include in the report
        :param groupings: How to group the report
        :param file_type: The type of file to generate
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param custom_name: The title of the report
        :param with_totals: Whether to include subtotals and grand totals in the report
        :param start_date: Start date of data to filter by
        :param end_date: End date of data to filter by
        :param dismissed_start_date: Dismissed start date of employees to filter by
        :param dismissed_end_date: Dismissed end date of employees to filter by
        :param payment_method: Payment method to filter by
        :param employment_type: Employee employment type to filter by
        :param employment_status: Employee employment status to filter by
        :param employee_uuids: Employees to filter by
        :param department_uuids: Departments to filter by
        :param work_address_uuids: Work addresses to filter by
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostCompaniesCompanyUUIDReportsRequest(
            company_uuid=company_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostCompaniesCompanyUUIDReportsRequestBody(
                columns=columns,
                groupings=groupings,
                custom_name=custom_name,
                file_type=file_type,
                with_totals=with_totals,
                start_date=start_date,
                end_date=end_date,
                dismissed_start_date=dismissed_start_date,
                dismissed_end_date=dismissed_end_date,
                payment_method=payment_method,
                employment_type=employment_type,
                employment_status=employment_status,
                employee_uuids=employee_uuids,
                department_uuids=department_uuids,
                work_address_uuids=work_address_uuids,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/companies/{company_uuid}/reports",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostCompaniesCompanyUUIDReportsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-companies-company_uuid-reports",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CreateReport)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        report_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Report:
        r"""Get a report

        Get a company's report given the `request_uuid`. The response will include the report request's status and, if complete, the report URL.

        scope: `company_reports:read`

        :param report_uuid: The UUID of the report request
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetReportsReportUUIDRequest(
            report_uuid=report_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/reports/{report_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-reports-report_uuid",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Report)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        report_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Report:
        r"""Get a report

        Get a company's report given the `request_uuid`. The response will include the report request's status and, if complete, the report URL.

        scope: `company_reports:read`

        :param report_uuid: The UUID of the report request
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetReportsReportUUIDRequest(
            report_uuid=report_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/reports/{report_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-reports-report_uuid",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Report)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get_template(
        self,
        *,
        company_uuid: str,
        report_type: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ReportTemplate:
        r"""Get a report template

        Get a company's report template. The only supported report type is `payroll_journal`. The resulting columns and groupings from this endpoint can be used as a guidance to create the report using the POST create report endpoint.

        scope: `company_reports:write`

        :param company_uuid: The UUID of the company
        :param report_type: The report type
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCompaniesCompanyUUIDReportTemplatesReportTypeRequest(
            company_uuid=company_uuid,
            report_type=report_type,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/companies/{company_uuid}/report_templates/{report_type}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-companies-company_uuid-report-templates-report_type",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ReportTemplate)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_template_async(
        self,
        *,
        company_uuid: str,
        report_type: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ReportTemplate:
        r"""Get a report template

        Get a company's report template. The only supported report type is `payroll_journal`. The resulting columns and groupings from this endpoint can be used as a guidance to create the report using the POST create report endpoint.

        scope: `company_reports:write`

        :param company_uuid: The UUID of the company
        :param report_type: The report type
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCompaniesCompanyUUIDReportTemplatesReportTypeRequest(
            company_uuid=company_uuid,
            report_type=report_type,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/companies/{company_uuid}/report_templates/{report_type}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-companies-company_uuid-report-templates-report_type",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ReportTemplate)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .company_address import CompanyAddress, CompanyAddressTypedDict
from enum import Enum
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class EntityType(str, Enum):
    r"""The tax payer type of the company."""

    C_CORPORATION = "C-Corporation"
    S_CORPORATION = "S-Corporation"
    SOLE_PROPRIETOR = "Sole proprietor"
    LLC = "LLC"
    LLP = "LLP"
    LIMITED_PARTNERSHIP = "Limited partnership"
    CO_OWNERSHIP = "Co-ownership"
    ASSOCIATION = "Association"
    TRUSTEESHIP = "Trusteeship"
    GENERAL_PARTNERSHIP = "General partnership"
    JOINT_VENTURE = "Joint venture"
    NON_PROFIT = "Non-Profit"


class Tier(str, Enum):
    r"""The Gusto product tier of the company (not applicable to Embedded partner managed companies)."""

    SIMPLE = "simple"
    PLUS = "plus"
    PREMIUM = "premium"
    CORE = "core"
    COMPLETE = "complete"
    CONCIERGE = "concierge"
    CONTRACTOR_ONLY = "contractor_only"
    BASIC = "basic"


class CompanyStatus(str, Enum):
    r"""The status of the company in Gusto. \"Approved\" companies are approved to run payroll from a risk and compliance perspective. However, an approved company may still need to resolve other [payroll blockers](https://docs.gusto.com/embedded-payroll/docs/payroll-blockers) to be able to run payroll. \"Not Approved\" companies may not yet run payroll with Gusto and may need to complete onboarding or contact support. \"Suspended\" companies may not run payroll with Gusto. In order to unsuspend their account, the company must contact support."""

    APPROVED = "Approved"
    NOT_APPROVED = "Not Approved"
    SUSPENDED = "Suspended"


class PayScheduleType(str, Enum):
    r"""The pay schedule assignment type."""

    SINGLE = "single"
    HOURLY_SALARIED = "hourly_salaried"
    BY_EMPLOYEE = "by_employee"
    BY_DEPARTMENT = "by_department"


class FundingType(str, Enum):
    r"""Company's default funding type"""

    ACH = "ach"
    REVERSE_WIRE = "reverse_wire"
    WIRE_IN = "wire_in"
    BREX = "brex"


class HourlyTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the hourly compensation rate."""
    multiple: NotRequired[float]
    r"""The amount multiplied by the base rate of a job to calculate compensation."""


class Hourly(BaseModel):
    name: Optional[str] = None
    r"""The name of the hourly compensation rate."""

    multiple: Optional[float] = None
    r"""The amount multiplied by the base rate of a job to calculate compensation."""


class FixedTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the fixed compensation."""


class Fixed(BaseModel):
    name: Optional[str] = None
    r"""The name of the fixed compensation."""


class CompanyPaidTimeOffTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the paid time off type."""


class CompanyPaidTimeOff(BaseModel):
    name: Optional[str] = None
    r"""The name of the paid time off type."""


class CompensationsTypedDict(TypedDict):
    r"""The available company-wide compensation rates for the company."""

    hourly: NotRequired[List[HourlyTypedDict]]
    r"""The available hourly compensation rates for the company."""
    fixed: NotRequired[List[FixedTypedDict]]
    r"""The available fixed compensation rates for the company."""
    paid_time_off: NotRequired[List[CompanyPaidTimeOffTypedDict]]
    r"""The available types of paid time off for the company."""


class Compensations(BaseModel):
    r"""The available company-wide compensation rates for the company."""

    hourly: Optional[List[Hourly]] = None
    r"""The available hourly compensation rates for the company."""

    fixed: Optional[List[Fixed]] = None
    r"""The available fixed compensation rates for the company."""

    paid_time_off: Optional[List[CompanyPaidTimeOff]] = None
    r"""The available types of paid time off for the company."""


class CompanyHomeAddressTypedDict(TypedDict):
    r"""The company's primary signatory's home address."""

    street_1: NotRequired[str]
    street_2: NotRequired[Nullable[str]]
    city: NotRequired[str]
    state: NotRequired[str]
    zip: NotRequired[str]
    country: NotRequired[str]


class CompanyHomeAddress(BaseModel):
    r"""The company's primary signatory's home address."""

    street_1: Optional[str] = None

    street_2: OptionalNullable[str] = UNSET

    city: Optional[str] = None

    state: Optional[str] = None

    zip: Optional[str] = None

    country: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["street_1", "street_2", "city", "state", "zip", "country"]
        nullable_fields = ["street_2"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PrimarySignatoryTypedDict(TypedDict):
    r"""The primary signatory of the company."""

    uuid: NotRequired[str]
    r"""The UUID of the company's primary signatory."""
    first_name: NotRequired[str]
    r"""The company's primary signatory's first name."""
    middle_initial: NotRequired[str]
    r"""The company's primary signatory's middle initial."""
    last_name: NotRequired[str]
    r"""The company's primary signatory's last name."""
    phone: NotRequired[str]
    r"""The company's primary signatory's phone number."""
    email: NotRequired[str]
    r"""The company's primary signatory's email address."""
    home_address: NotRequired[CompanyHomeAddressTypedDict]
    r"""The company's primary signatory's home address."""


class PrimarySignatory(BaseModel):
    r"""The primary signatory of the company."""

    uuid: Optional[str] = None
    r"""The UUID of the company's primary signatory."""

    first_name: Optional[str] = None
    r"""The company's primary signatory's first name."""

    middle_initial: Optional[str] = None
    r"""The company's primary signatory's middle initial."""

    last_name: Optional[str] = None
    r"""The company's primary signatory's last name."""

    phone: Optional[str] = None
    r"""The company's primary signatory's phone number."""

    email: Optional[str] = None
    r"""The company's primary signatory's email address."""

    home_address: Optional[CompanyHomeAddress] = None
    r"""The company's primary signatory's home address."""


class PrimaryPayrollAdminTypedDict(TypedDict):
    r"""The primary payroll admin of the company."""

    first_name: NotRequired[str]
    r"""The company's primary payroll admin's first name."""
    last_name: NotRequired[str]
    r"""The company's primary payroll admin's last name."""
    phone: NotRequired[Nullable[str]]
    r"""The company's primary payroll admin's phone number."""
    email: NotRequired[str]
    r"""The company's primary payroll admin's email address."""


class PrimaryPayrollAdmin(BaseModel):
    r"""The primary payroll admin of the company."""

    first_name: Optional[str] = None
    r"""The company's primary payroll admin's first name."""

    last_name: Optional[str] = None
    r"""The company's primary payroll admin's last name."""

    phone: OptionalNullable[str] = UNSET
    r"""The company's primary payroll admin's phone number."""

    email: Optional[str] = None
    r"""The company's primary payroll admin's email address."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["first_name", "last_name", "phone", "email"]
        nullable_fields = ["phone"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CompanyTypedDict(TypedDict):
    r"""The representation of a company in Gusto."""

    uuid: str
    r"""A unique identifier of the company in Gusto."""
    ein: NotRequired[str]
    r"""The Federal Employer Identification Number of the company."""
    entity_type: NotRequired[Nullable[EntityType]]
    r"""The tax payer type of the company."""
    contractor_only: NotRequired[bool]
    r"""Whether the company only supports contractors."""
    tier: NotRequired[Nullable[Tier]]
    r"""The Gusto product tier of the company (not applicable to Embedded partner managed companies)."""
    is_suspended: NotRequired[bool]
    r"""Whether or not the company is suspended in Gusto. Suspended companies may not run payroll."""
    company_status: NotRequired[CompanyStatus]
    r"""The status of the company in Gusto. \"Approved\" companies are approved to run payroll from a risk and compliance perspective. However, an approved company may still need to resolve other [payroll blockers](https://docs.gusto.com/embedded-payroll/docs/payroll-blockers) to be able to run payroll. \"Not Approved\" companies may not yet run payroll with Gusto and may need to complete onboarding or contact support. \"Suspended\" companies may not run payroll with Gusto. In order to unsuspend their account, the company must contact support."""
    name: NotRequired[str]
    r"""The name of the company."""
    slug: NotRequired[str]
    r"""The slug of the name of the company."""
    trade_name: NotRequired[Nullable[str]]
    r"""The trade name of the company."""
    is_partner_managed: NotRequired[bool]
    r"""Whether the company is fully managed by a partner via the API"""
    pay_schedule_type: NotRequired[Nullable[PayScheduleType]]
    r"""The pay schedule assignment type."""
    join_date: NotRequired[Nullable[str]]
    r"""Company's first invoiceable event date"""
    funding_type: NotRequired[Nullable[FundingType]]
    r"""Company's default funding type"""
    locations: NotRequired[List[CompanyAddressTypedDict]]
    r"""The locations of the company."""
    compensations: NotRequired[CompensationsTypedDict]
    r"""The available company-wide compensation rates for the company."""
    primary_signatory: NotRequired[Nullable[PrimarySignatoryTypedDict]]
    r"""The primary signatory of the company."""
    primary_payroll_admin: NotRequired[PrimaryPayrollAdminTypedDict]
    r"""The primary payroll admin of the company."""


class Company(BaseModel):
    r"""The representation of a company in Gusto."""

    uuid: str
    r"""A unique identifier of the company in Gusto."""

    ein: Optional[str] = None
    r"""The Federal Employer Identification Number of the company."""

    entity_type: OptionalNullable[EntityType] = UNSET
    r"""The tax payer type of the company."""

    contractor_only: Optional[bool] = None
    r"""Whether the company only supports contractors."""

    tier: OptionalNullable[Tier] = UNSET
    r"""The Gusto product tier of the company (not applicable to Embedded partner managed companies)."""

    is_suspended: Optional[bool] = None
    r"""Whether or not the company is suspended in Gusto. Suspended companies may not run payroll."""

    company_status: Optional[CompanyStatus] = None
    r"""The status of the company in Gusto. \"Approved\" companies are approved to run payroll from a risk and compliance perspective. However, an approved company may still need to resolve other [payroll blockers](https://docs.gusto.com/embedded-payroll/docs/payroll-blockers) to be able to run payroll. \"Not Approved\" companies may not yet run payroll with Gusto and may need to complete onboarding or contact support. \"Suspended\" companies may not run payroll with Gusto. In order to unsuspend their account, the company must contact support."""

    name: Optional[str] = None
    r"""The name of the company."""

    slug: Optional[str] = None
    r"""The slug of the name of the company."""

    trade_name: OptionalNullable[str] = UNSET
    r"""The trade name of the company."""

    is_partner_managed: Optional[bool] = None
    r"""Whether the company is fully managed by a partner via the API"""

    pay_schedule_type: OptionalNullable[PayScheduleType] = UNSET
    r"""The pay schedule assignment type."""

    join_date: OptionalNullable[str] = UNSET
    r"""Company's first invoiceable event date"""

    funding_type: OptionalNullable[FundingType] = UNSET
    r"""Company's default funding type"""

    locations: Optional[List[CompanyAddress]] = None
    r"""The locations of the company."""

    compensations: Optional[Compensations] = None
    r"""The available company-wide compensation rates for the company."""

    primary_signatory: OptionalNullable[PrimarySignatory] = UNSET
    r"""The primary signatory of the company."""

    primary_payroll_admin: Optional[PrimaryPayrollAdmin] = None
    r"""The primary payroll admin of the company."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "ein",
            "entity_type",
            "contractor_only",
            "tier",
            "is_suspended",
            "company_status",
            "name",
            "slug",
            "trade_name",
            "is_partner_managed",
            "pay_schedule_type",
            "join_date",
            "funding_type",
            "locations",
            "compensations",
            "primary_signatory",
            "primary_payroll_admin",
        ]
        nullable_fields = [
            "entity_type",
            "tier",
            "trade_name",
            "pay_schedule_type",
            "join_date",
            "funding_type",
            "primary_signatory",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ExternalPayrollBasicStatus(str, Enum):
    r"""The status of the external payroll. The status will be `unprocessed` when the external payroll is created and transition to `processed` once tax liabilities are entered and finalized.  Once in the `processed` status all actions that can edit an external payroll will be disabled."""

    UNPROCESSED = "unprocessed"
    PROCESSED = "processed"


class ExternalPayrollBasicTypedDict(TypedDict):
    r"""The representation of an external payroll with minimal information."""

    uuid: str
    r"""The UUID of the external payroll."""
    company_uuid: NotRequired[str]
    r"""The UUID of the company."""
    check_date: NotRequired[str]
    r"""External payroll's check date."""
    payment_period_start_date: NotRequired[str]
    r"""External payroll's pay period start date."""
    payment_period_end_date: NotRequired[str]
    r"""External payroll's pay period end date."""
    status: NotRequired[ExternalPayrollBasicStatus]
    r"""The status of the external payroll. The status will be `unprocessed` when the external payroll is created and transition to `processed` once tax liabilities are entered and finalized.  Once in the `processed` status all actions that can edit an external payroll will be disabled."""


class ExternalPayrollBasic(BaseModel):
    r"""The representation of an external payroll with minimal information."""

    uuid: str
    r"""The UUID of the external payroll."""

    company_uuid: Optional[str] = None
    r"""The UUID of the company."""

    check_date: Optional[str] = None
    r"""External payroll's check date."""

    payment_period_start_date: Optional[str] = None
    r"""External payroll's pay period start date."""

    payment_period_end_date: Optional[str] = None
    r"""External payroll's pay period end date."""

    status: Optional[ExternalPayrollBasicStatus] = None
    r"""The status of the external payroll. The status will be `unprocessed` when the external payroll is created and transition to `processed` once tax liabilities are entered and finalized.  Once in the `processed` status all actions that can edit an external payroll will be disabled."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from gusto_embedded.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class W4DataType(str, Enum):
    r"""The version of w4 form."""

    PRE_2020_W4 = "pre_2020_w4"
    REV_2020_W4 = "rev_2020_w4"


class EmployeeFederalTaxTypedDict(TypedDict):
    r"""Example response"""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    filing_status: str
    r"""It determines which tax return form an individual will use and is an important factor in computing taxable income. One of:
    - Single
    - Married
    - Head of Household
    - Exempt from withholding
    - Married, but withhold as Single (does not apply to rev_2020_w4 form)
    """
    extra_withholding: Nullable[str]
    r"""An employee can request an additional amount to be withheld from each paycheck."""
    two_jobs: Nullable[bool]
    r"""If there are only two jobs (i.e., you and your spouse each have a job, or you have two), you can set it to true."""
    dependents_amount: Nullable[str]
    r"""A dependent is a person other than the taxpayer or spouse who entitles the taxpayer to claim a dependency exemption."""
    other_income: Nullable[str]
    r"""Other income amount."""
    deductions: Nullable[str]
    r"""Deductions other than the standard deduction to reduce withholding."""
    w4_data_type: W4DataType
    r"""The version of w4 form."""
    federal_withholding_allowance: NotRequired[float]
    r"""*does not apply to rev_2020_w4 form*

    An exemption from paying a certain amount of income tax.
    """
    additional_withholding: NotRequired[bool]
    r"""*does not apply to rev_2020_w4 form*"""


class EmployeeFederalTax(BaseModel):
    r"""Example response"""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    filing_status: str
    r"""It determines which tax return form an individual will use and is an important factor in computing taxable income. One of:
    - Single
    - Married
    - Head of Household
    - Exempt from withholding
    - Married, but withhold as Single (does not apply to rev_2020_w4 form)
    """

    extra_withholding: Nullable[str]
    r"""An employee can request an additional amount to be withheld from each paycheck."""

    two_jobs: Nullable[bool]
    r"""If there are only two jobs (i.e., you and your spouse each have a job, or you have two), you can set it to true."""

    dependents_amount: Nullable[str]
    r"""A dependent is a person other than the taxpayer or spouse who entitles the taxpayer to claim a dependency exemption."""

    other_income: Nullable[str]
    r"""Other income amount."""

    deductions: Nullable[str]
    r"""Deductions other than the standard deduction to reduce withholding."""

    w4_data_type: W4DataType
    r"""The version of w4 form."""

    federal_withholding_allowance: Optional[float] = None
    r"""*does not apply to rev_2020_w4 form*

    An exemption from paying a certain amount of income tax.
    """

    additional_withholding: Optional[bool] = None
    r"""*does not apply to rev_2020_w4 form*"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["federal_withholding_allowance", "additional_withholding"]
        nullable_fields = [
            "extra_withholding",
            "two_jobs",
            "dependents_amount",
            "other_income",
            "deductions",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ResourcesTypedDict(TypedDict):
    entity_type: str
    r"""The type of entity being described, could be “Contractor”, “Employee”, “BankAccount”, “Payroll”, “ContractorPayment”, “RecoveryCase”, or “Signatory”"""
    entity_uuid: str
    r"""Unique identifier of the entity"""
    reference_type: NotRequired[str]
    r"""Optional. The type of a resource that is related to the one described by entity_type and entity_uuid. For instance, if the entity_type is “BankAccount”, the reference_type could be the “Employee” or “Contractor” to whom the bank account belongs."""
    reference_uuid: NotRequired[str]
    r"""Optional. Unique identifier of the reference."""


class Resources(BaseModel):
    entity_type: str
    r"""The type of entity being described, could be “Contractor”, “Employee”, “BankAccount”, “Payroll”, “ContractorPayment”, “RecoveryCase”, or “Signatory”"""

    entity_uuid: str
    r"""Unique identifier of the entity"""

    reference_type: Optional[str] = None
    r"""Optional. The type of a resource that is related to the one described by entity_type and entity_uuid. For instance, if the entity_type is “BankAccount”, the reference_type could be the “Employee” or “Contractor” to whom the bank account belongs."""

    reference_uuid: Optional[str] = None
    r"""Optional. Unique identifier of the reference."""


class NotificationTypedDict(TypedDict):
    r"""Representation of a notification"""

    uuid: str
    r"""Unique identifier of a notification."""
    company_uuid: NotRequired[str]
    r"""Unique identifier of the company to which the notification belongs."""
    title: NotRequired[str]
    r"""The title of the notification. This highlights the actionable component of the notification."""
    message: NotRequired[str]
    r"""The message of the notification. This provides additional context for the user and recommends a specific action to resolve the notification."""
    category: NotRequired[str]
    r"""The notification's category."""
    actionable: NotRequired[bool]
    r"""Indicates whether a notification requires action or not. If false, the notification provides critical information only."""
    published_at: NotRequired[str]
    r"""Timestamp of when the notification was published."""
    due_at: NotRequired[str]
    r"""Timestamp of when the notification is due. If the notification has no due date, this field will be null."""
    resources: NotRequired[List[ResourcesTypedDict]]
    r"""An array of entities relevant to the notification"""


class Notification(BaseModel):
    r"""Representation of a notification"""

    uuid: str
    r"""Unique identifier of a notification."""

    company_uuid: Optional[str] = None
    r"""Unique identifier of the company to which the notification belongs."""

    title: Optional[str] = None
    r"""The title of the notification. This highlights the actionable component of the notification."""

    message: Optional[str] = None
    r"""The message of the notification. This provides additional context for the user and recommends a specific action to resolve the notification."""

    category: Optional[str] = None
    r"""The notification's category."""

    actionable: Optional[bool] = None
    r"""Indicates whether a notification requires action or not. If false, the notification provides critical information only."""

    published_at: Optional[str] = None
    r"""Timestamp of when the notification was published."""

    due_at: Optional[str] = None
    r"""Timestamp of when the notification is due. If the notification has no due date, this field will be null."""

    resources: Optional[List[Resources]] = None
    r"""An array of entities relevant to the notification"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PayrollCompanyTaxesTypeTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The tax name"""
    employer: NotRequired[bool]
    r"""Whether this tax is an employer or employee tax"""
    amount: NotRequired[str]
    r"""The amount of this tax for the payroll"""


class PayrollCompanyTaxesType(BaseModel):
    name: Optional[str] = None
    r"""The tax name"""

    employer: Optional[bool] = None
    r"""Whether this tax is an employer or employee tax"""

    amount: Optional[str] = None
    r"""The amount of this tax for the payroll"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import date
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EmployeeAddressTypedDict(TypedDict):
    version: NotRequired[str]
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    street_1: NotRequired[str]
    street_2: NotRequired[Nullable[str]]
    city: NotRequired[str]
    state: NotRequired[str]
    zip: NotRequired[str]
    country: NotRequired[str]
    active: NotRequired[bool]
    r"""The status of the location. Inactive locations have been deleted, but may still have historical data associated with them."""
    uuid: NotRequired[str]
    r"""The UUID of the employee address"""
    employee_uuid: NotRequired[str]
    r"""The UUID of the employee"""
    effective_date: NotRequired[date]
    r"""The date the employee started living at the address."""
    courtesy_withholding: NotRequired[bool]
    r"""Determines if home taxes should be withheld and paid for employee."""


class EmployeeAddress(BaseModel):
    version: Optional[str] = None
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    street_1: Optional[str] = None

    street_2: OptionalNullable[str] = UNSET

    city: Optional[str] = None

    state: Optional[str] = None

    zip: Optional[str] = None

    country: Optional[str] = "USA"

    active: Optional[bool] = None
    r"""The status of the location. Inactive locations have been deleted, but may still have historical data associated with them."""

    uuid: Optional[str] = None
    r"""The UUID of the employee address"""

    employee_uuid: Optional[str] = None
    r"""The UUID of the employee"""

    effective_date: Optional[date] = None
    r"""The date the employee started living at the address."""

    courtesy_withholding: Optional[bool] = None
    r"""Determines if home taxes should be withheld and paid for employee."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "version",
            "street_1",
            "street_2",
            "city",
            "state",
            "zip",
            "country",
            "active",
            "uuid",
            "employee_uuid",
            "effective_date",
            "courtesy_withholding",
        ]
        nullable_fields = ["street_2"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class AccountType(str, Enum):
    r"""Bank account type"""

    CHECKING = "Checking"
    SAVINGS = "Savings"


class VerificationStatus(str, Enum):
    r"""The verification status of the bank account.

    'awaiting_deposits' means the bank account is just created and money is being transferred.
    'ready_for_verification' means the micro-deposits are completed and the verification process can begin by using the verify endpoint.
    'verified' means the bank account is verified.
    """

    AWAITING_DEPOSITS = "awaiting_deposits"
    READY_FOR_VERIFICATION = "ready_for_verification"
    VERIFIED = "verified"


class VerificationType(str, Enum):
    r"""The verification type of the bank account.

    'bank_deposits' means the bank account is connected by entering routing and accounting numbers and verifying through micro-deposits.
    'plaid' means the bank account is connected through Plaid.
    """

    BANK_DEPOSITS = "bank_deposits"
    PLAID = "plaid"
    PLAID_EXTERNAL = "plaid_external"


class PlaidStatus(str, Enum):
    r"""The Plaid connection status of the bank account. Only applies when verification type is Plaid."""

    CONNECTED = "connected"
    DISCONNECTED = "disconnected"


class CompanyBankAccountTypedDict(TypedDict):
    r"""The company bank account"""

    uuid: str
    r"""UUID of the bank account"""
    company_uuid: NotRequired[str]
    r"""UUID of the company"""
    account_type: NotRequired[AccountType]
    r"""Bank account type"""
    routing_number: NotRequired[str]
    r"""The bank account's routing number"""
    hidden_account_number: NotRequired[str]
    r"""Masked bank account number"""
    verification_status: NotRequired[VerificationStatus]
    r"""The verification status of the bank account.

    'awaiting_deposits' means the bank account is just created and money is being transferred.
    'ready_for_verification' means the micro-deposits are completed and the verification process can begin by using the verify endpoint.
    'verified' means the bank account is verified.
    """
    verification_type: NotRequired[VerificationType]
    r"""The verification type of the bank account.

    'bank_deposits' means the bank account is connected by entering routing and accounting numbers and verifying through micro-deposits.
    'plaid' means the bank account is connected through Plaid.
    """
    plaid_status: NotRequired[Nullable[PlaidStatus]]
    r"""The Plaid connection status of the bank account. Only applies when verification type is Plaid."""
    last_cached_balance: NotRequired[Nullable[str]]
    r"""The last fetch balance for the bank account. Please be aware that this amount does not reflect the most up-to-date balance and only applies when the verification type is Plaid."""
    balance_fetched_date: NotRequired[Nullable[str]]
    r"""The balance fetch date associated with the last_cached_balance. Only applies when verification type is Plaid."""
    name: NotRequired[str]
    r"""Name of bank account"""


class CompanyBankAccount(BaseModel):
    r"""The company bank account"""

    uuid: str
    r"""UUID of the bank account"""

    company_uuid: Optional[str] = None
    r"""UUID of the company"""

    account_type: Optional[AccountType] = None
    r"""Bank account type"""

    routing_number: Optional[str] = None
    r"""The bank account's routing number"""

    hidden_account_number: Optional[str] = None
    r"""Masked bank account number"""

    verification_status: Optional[VerificationStatus] = None
    r"""The verification status of the bank account.

    'awaiting_deposits' means the bank account is just created and money is being transferred.
    'ready_for_verification' means the micro-deposits are completed and the verification process can begin by using the verify endpoint.
    'verified' means the bank account is verified.
    """

    verification_type: Optional[VerificationType] = None
    r"""The verification type of the bank account.

    'bank_deposits' means the bank account is connected by entering routing and accounting numbers and verifying through micro-deposits.
    'plaid' means the bank account is connected through Plaid.
    """

    plaid_status: OptionalNullable[PlaidStatus] = UNSET
    r"""The Plaid connection status of the bank account. Only applies when verification type is Plaid."""

    last_cached_balance: OptionalNullable[str] = UNSET
    r"""The last fetch balance for the bank account. Please be aware that this amount does not reflect the most up-to-date balance and only applies when the verification type is Plaid."""

    balance_fetched_date: OptionalNullable[str] = UNSET
    r"""The balance fetch date associated with the last_cached_balance. Only applies when verification type is Plaid."""

    name: Optional[str] = None
    r"""Name of bank account"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "company_uuid",
            "account_type",
            "routing_number",
            "hidden_account_number",
            "verification_status",
            "verification_type",
            "plaid_status",
            "last_cached_balance",
            "balance_fetched_date",
            "name",
        ]
        nullable_fields = [
            "plaid_status",
            "last_cached_balance",
            "balance_fetched_date",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutCompaniesCompanyUUIDHolidayPayPolicyNewYearsDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyNewYearsDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyMlkDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyMlkDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyPresidentsDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyPresidentsDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyMemorialDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyMemorialDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyJuneteenthTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyJuneteenth(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyIndependenceDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyIndependenceDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyLaborDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyLaborDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyColumbusDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyColumbusDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyVeteransDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyVeteransDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyThanksgivingTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyThanksgiving(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyChristmasDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PutCompaniesCompanyUUIDHolidayPayPolicyChristmasDay(BaseModel):
    selected: Optional[bool] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyFederalHolidaysTypedDict(TypedDict):
    r"""An object containing federal holiday objects, each containing a boolean selected property."""

    new_years_day: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyNewYearsDayTypedDict
    ]
    mlk_day: NotRequired[PutCompaniesCompanyUUIDHolidayPayPolicyMlkDayTypedDict]
    presidents_day: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyPresidentsDayTypedDict
    ]
    memorial_day: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyMemorialDayTypedDict
    ]
    juneteenth: NotRequired[PutCompaniesCompanyUUIDHolidayPayPolicyJuneteenthTypedDict]
    independence_day: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyIndependenceDayTypedDict
    ]
    labor_day: NotRequired[PutCompaniesCompanyUUIDHolidayPayPolicyLaborDayTypedDict]
    columbus_day: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyColumbusDayTypedDict
    ]
    veterans_day: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyVeteransDayTypedDict
    ]
    thanksgiving: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyThanksgivingTypedDict
    ]
    christmas_day: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyChristmasDayTypedDict
    ]


class PutCompaniesCompanyUUIDHolidayPayPolicyFederalHolidays(BaseModel):
    r"""An object containing federal holiday objects, each containing a boolean selected property."""

    new_years_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyNewYearsDay] = None

    mlk_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyMlkDay] = None

    presidents_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyPresidentsDay] = (
        None
    )

    memorial_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyMemorialDay] = None

    juneteenth: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyJuneteenth] = None

    independence_day: Optional[
        PutCompaniesCompanyUUIDHolidayPayPolicyIndependenceDay
    ] = None

    labor_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyLaborDay] = None

    columbus_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyColumbusDay] = None

    veterans_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyVeteransDay] = None

    thanksgiving: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyThanksgiving] = None

    christmas_day: Optional[PutCompaniesCompanyUUIDHolidayPayPolicyChristmasDay] = None


class PutCompaniesCompanyUUIDHolidayPayPolicyRequestBodyTypedDict(TypedDict):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    federal_holidays: NotRequired[
        PutCompaniesCompanyUUIDHolidayPayPolicyFederalHolidaysTypedDict
    ]
    r"""An object containing federal holiday objects, each containing a boolean selected property."""


class PutCompaniesCompanyUUIDHolidayPayPolicyRequestBody(BaseModel):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    federal_holidays: Optional[
        PutCompaniesCompanyUUIDHolidayPayPolicyFederalHolidays
    ] = None
    r"""An object containing federal holiday objects, each containing a boolean selected property."""


class PutCompaniesCompanyUUIDHolidayPayPolicyRequestTypedDict(TypedDict):
    company_uuid: str
    r"""The UUID of the company"""
    request_body: PutCompaniesCompanyUUIDHolidayPayPolicyRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutCompaniesCompanyUUIDHolidayPayPolicyRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PutCompaniesCompanyUUIDHolidayPayPolicyRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import date
from gusto_embedded import models, utils
from gusto_embedded._hooks import HookContext
from gusto_embedded.types import OptionalNullable, UNSET
from gusto_embedded.utils import get_security_from_env
from typing import Any, List, Mapping, Optional, Union


class ContractorPaymentGroups(BaseSDK):
    def create(
        self,
        *,
        company_id: str,
        check_date: date,
        contractor_payments: Union[
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsContractorPayments
            ],
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsContractorPaymentsTypedDict
            ],
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        creation_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Create a contractor payment group

        Pay a group of contractors. Information needed depends on the contractor's wage type (hourly vs fixed)

        scope: `payrolls:run`

        :param company_id: The UUID of the company
        :param check_date: The payment check date
        :param contractor_payments:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param creation_token: Optional token used to make contractor payment group creation idempotent.  If provided, string must be unique for each group you intend to create.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostV1CompaniesCompanyIDContractorPaymentGroupsRequest(
            company_id=company_id,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostV1CompaniesCompanyIDContractorPaymentGroupsRequestBody(
                check_date=check_date,
                creation_token=creation_token,
                contractor_payments=utils.get_pydantic_model(
                    contractor_payments,
                    List[
                        models.PostV1CompaniesCompanyIDContractorPaymentGroupsContractorPayments
                    ],
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/companies/{company_id}/contractor_payment_groups",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-v1-companies-company_id-contractor_payment_groups",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_async(
        self,
        *,
        company_id: str,
        check_date: date,
        contractor_payments: Union[
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsContractorPayments
            ],
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsContractorPaymentsTypedDict
            ],
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        creation_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Create a contractor payment group

        Pay a group of contractors. Information needed depends on the contractor's wage type (hourly vs fixed)

        scope: `payrolls:run`

        :param company_id: The UUID of the company
        :param check_date: The payment check date
        :param contractor_payments:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param creation_token: Optional token used to make contractor payment group creation idempotent.  If provided, string must be unique for each group you intend to create.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostV1CompaniesCompanyIDContractorPaymentGroupsRequest(
            company_id=company_id,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostV1CompaniesCompanyIDContractorPaymentGroupsRequestBody(
                check_date=check_date,
                creation_token=creation_token,
                contractor_payments=utils.get_pydantic_model(
                    contractor_payments,
                    List[
                        models.PostV1CompaniesCompanyIDContractorPaymentGroupsContractorPayments
                    ],
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/companies/{company_id}/contractor_payment_groups",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-v1-companies-company_id-contractor_payment_groups",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get_list(
        self,
        *,
        company_id: str,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        page: Optional[int] = None,
        per: Optional[int] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.ContractorPaymentGroupMinimal]:
        r"""Get contractor payment groups for a company

        Returns a list of minimal contractor payment groups within a given time period, including totals but not associated contractor payments.

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param start_date: The time period for which to retrieve contractor payment groups. Defaults to 6 months ago.
        :param end_date: The time period for which to retrieve contractor payment groups. Defaults to today's date.
        :param page: The page that is requested. When unspecified, will load all objects unless endpoint forces pagination.
        :param per: Number of objects per page. For majority of endpoints will default to 25
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1CompaniesCompanyIDContractorPaymentGroupsRequest(
            company_id=company_id,
            start_date=start_date,
            end_date=end_date,
            page=page,
            per=per,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/companies/{company_id}/contractor_payment_groups",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-companies-company_id-contractor_payment_groups",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, List[models.ContractorPaymentGroupMinimal]
            )
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_list_async(
        self,
        *,
        company_id: str,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        page: Optional[int] = None,
        per: Optional[int] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.ContractorPaymentGroupMinimal]:
        r"""Get contractor payment groups for a company

        Returns a list of minimal contractor payment groups within a given time period, including totals but not associated contractor payments.

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param start_date: The time period for which to retrieve contractor payment groups. Defaults to 6 months ago.
        :param end_date: The time period for which to retrieve contractor payment groups. Defaults to today's date.
        :param page: The page that is requested. When unspecified, will load all objects unless endpoint forces pagination.
        :param per: Number of objects per page. For majority of endpoints will default to 25
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1CompaniesCompanyIDContractorPaymentGroupsRequest(
            company_id=company_id,
            start_date=start_date,
            end_date=end_date,
            page=page,
            per=per,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/companies/{company_id}/contractor_payment_groups",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-companies-company_id-contractor_payment_groups",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, List[models.ContractorPaymentGroupMinimal]
            )
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def preview(
        self,
        *,
        company_id: str,
        check_date: date,
        contractor_payments: Union[
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewContractorPayments
            ],
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewContractorPaymentsTypedDict
            ],
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        creation_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Preview a contractor payment group

        Preview a group of contractor payments. Request will validate inputs and return preview of the contractor payment group including the expected debit_date.  Uuid will be null in the response.

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param check_date: The payment check date
        :param contractor_payments:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param creation_token: Optional token used to make contractor payment group creation idempotent.  If provided, string must be unique for each group you intend to create.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewRequest(
            company_id=company_id,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewRequestBody(
                check_date=check_date,
                creation_token=creation_token,
                contractor_payments=utils.get_pydantic_model(
                    contractor_payments,
                    List[
                        models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewContractorPayments
                    ],
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/companies/{company_id}/contractor_payment_groups/preview",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-v1-companies-company_id-contractor_payment_groups-preview",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def preview_async(
        self,
        *,
        company_id: str,
        check_date: date,
        contractor_payments: Union[
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewContractorPayments
            ],
            List[
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewContractorPaymentsTypedDict
            ],
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        creation_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Preview a contractor payment group

        Preview a group of contractor payments. Request will validate inputs and return preview of the contractor payment group including the expected debit_date.  Uuid will be null in the response.

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param check_date: The payment check date
        :param contractor_payments:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param creation_token: Optional token used to make contractor payment group creation idempotent.  If provided, string must be unique for each group you intend to create.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewRequest(
            company_id=company_id,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewRequestBody(
                check_date=check_date,
                creation_token=creation_token,
                contractor_payments=utils.get_pydantic_model(
                    contractor_payments,
                    List[
                        models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewContractorPayments
                    ],
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/companies/{company_id}/contractor_payment_groups/preview",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostV1CompaniesCompanyIDContractorPaymentGroupsPreviewRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-v1-companies-company_id-contractor_payment_groups-preview",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        contractor_payment_group_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Fetch a contractor payment group

        Returns a contractor payment group with all associated contractor payments.

        scope: `payrolls:read`

        :param contractor_payment_group_uuid: The UUID of the contractor payment group
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1ContractorPaymentGroupsContractorPaymentGroupIDRequest(
            contractor_payment_group_uuid=contractor_payment_group_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/contractor_payment_groups/{contractor_payment_group_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-contractor_payment_groups-contractor_payment_group_id",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        contractor_payment_group_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Fetch a contractor payment group

        Returns a contractor payment group with all associated contractor payments.

        scope: `payrolls:read`

        :param contractor_payment_group_uuid: The UUID of the contractor payment group
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1ContractorPaymentGroupsContractorPaymentGroupIDRequest(
            contractor_payment_group_uuid=contractor_payment_group_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/contractor_payment_groups/{contractor_payment_group_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-contractor_payment_groups-contractor_payment_group_id",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        contractor_payment_group_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Cancel a contractor payment group

        Cancels a contractor payment group and all associated contractor payments. All contractor payments must be cancellable, unfunded.

        scope: `payrolls:run`

        :param contractor_payment_group_uuid: The UUID of the contractor payment group
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteV1ContractorPaymentGroupsContractorPaymentGroupIDRequest(
            contractor_payment_group_uuid=contractor_payment_group_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="DELETE",
            path="/v1/contractor_payment_groups/{contractor_payment_group_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="delete-v1-contractor_payment_groups-contractor_payment_group_id",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        contractor_payment_group_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Cancel a contractor payment group

        Cancels a contractor payment group and all associated contractor payments. All contractor payments must be cancellable, unfunded.

        scope: `payrolls:run`

        :param contractor_payment_group_uuid: The UUID of the contractor payment group
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteV1ContractorPaymentGroupsContractorPaymentGroupIDRequest(
            contractor_payment_group_uuid=contractor_payment_group_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v1/contractor_payment_groups/{contractor_payment_group_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="delete-v1-contractor_payment_groups-contractor_payment_group_id",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def fund(
        self,
        *,
        contractor_payment_group_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Fund a contractor payment group [DEMO]

        > 🚧 Demo action
        >
        > This action is only available in the Demo environment

        Simulate funding a contractor payment group. Funding only occurs automatically in the production environment when bank transactions are generated. Use this action in the demo environment to transition a contractor payment group's `status` from `Unfunded` to `Funded`. A `Funded` status is required for generating a contractor payment receipt.

        scope: `payrolls:run`

        :param contractor_payment_group_uuid: The UUID of the contractor payment group
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.PutV1ContractorPaymentGroupsContractorPaymentGroupIDFundRequest(
                contractor_payment_group_uuid=contractor_payment_group_uuid,
                x_gusto_api_version=x_gusto_api_version,
            )
        )

        req = self._build_request(
            method="PUT",
            path="/v1/contractor_payment_groups/{contractor_payment_group_uuid}/fund",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-contractor_payment_groups-contractor_payment_group_id-fund",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def fund_async(
        self,
        *,
        contractor_payment_group_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ContractorPaymentGroup:
        r"""Fund a contractor payment group [DEMO]

        > 🚧 Demo action
        >
        > This action is only available in the Demo environment

        Simulate funding a contractor payment group. Funding only occurs automatically in the production environment when bank transactions are generated. Use this action in the demo environment to transition a contractor payment group's `status` from `Unfunded` to `Funded`. A `Funded` status is required for generating a contractor payment receipt.

        scope: `payrolls:run`

        :param contractor_payment_group_uuid: The UUID of the contractor payment group
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.PutV1ContractorPaymentGroupsContractorPaymentGroupIDFundRequest(
                contractor_payment_group_uuid=contractor_payment_group_uuid,
                x_gusto_api_version=x_gusto_api_version,
            )
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/contractor_payment_groups/{contractor_payment_group_uuid}/fund",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-contractor_payment_groups-contractor_payment_group_id-fund",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ContractorPaymentGroup)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ContractorBankAccountAccountType(str, Enum):
    r"""Bank account type"""

    CHECKING = "Checking"
    SAVINGS = "Savings"


class ContractorBankAccountTypedDict(TypedDict):
    r"""Example response"""

    uuid: str
    r"""UUID of the bank account"""
    contractor_uuid: NotRequired[str]
    r"""UUID of the employee"""
    account_type: NotRequired[ContractorBankAccountAccountType]
    r"""Bank account type"""
    name: NotRequired[str]
    r"""Name for the bank account"""
    routing_number: NotRequired[str]
    r"""The bank account's routing number"""
    hidden_account_number: NotRequired[str]
    r"""Masked bank account number"""


class ContractorBankAccount(BaseModel):
    r"""Example response"""

    uuid: str
    r"""UUID of the bank account"""

    contractor_uuid: Optional[str] = None
    r"""UUID of the employee"""

    account_type: Optional[ContractorBankAccountAccountType] = None
    r"""Bank account type"""

    name: Optional[str] = None
    r"""Name for the bank account"""

    routing_number: Optional[str] = None
    r"""The bank account's routing number"""

    hidden_account_number: Optional[str] = None
    r"""Masked bank account number"""
